@@ -1,7 +1,7 @@
 /*
  * trigger-callback.c - callbacks for triggers
  *
- * Copyright (C) 2014-2022 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2014-2024 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -147,7 +147,7 @@ trigger_callback_set_common_vars (struct t_trigger *trigger,
     weechat_hashtable_set (
         hashtable, "tg_hook_type",
         trigger_hook_type_string[
-            weechat_config_integer (trigger->options[TRIGGER_OPTION_HOOK])]);
+            weechat_config_enum (trigger->options[TRIGGER_OPTION_HOOK])]);
 }
 
 /*
@@ -210,21 +210,32 @@ trigger_callback_set_tags (struct t_gui_buffer *buffer,
         {
             /*
              * example:
-             *   tag: "irc_tag_time_2021-12-30T21:02:50.038Z"
+             *   tag: "irc_tag_time=2021-12-30T21:02:50.038Z"
              * is added in the hashtable like this:
              *   key  : "tg_tag_irc_time"
              *   value: "2021-12-30T21:02:50.038Z"
              */
-            pos = strchr (tags[i] + 8, '_');
-            if (pos && pos > tags[i] + 8)
+            pos = strchr (tags[i] + 8, '=');
+            if (!pos || (pos > tags[i] + 8))
             {
-                key = weechat_strndup (tags[i] + 8, pos - tags[i] - 8);
-                if (key)
+                if (pos)
                 {
+                    /* tag with value */
+                    key = weechat_strndup (tags[i] + 8, pos - tags[i] - 8);
+                    if (key)
+                    {
+                        snprintf (str_temp, sizeof (str_temp),
+                                  "tg_tag_irc_%s", key);
+                        weechat_hashtable_set (extra_vars, str_temp, pos + 1);
+                        free (key);
+                    }
+                }
+                else
+                {
+                    /* tag without value */
                     snprintf (str_temp, sizeof (str_temp),
-                              "tg_tag_irc_%s", key);
-                    weechat_hashtable_set (extra_vars, str_temp, pos + 1);
-                    free (key);
+                              "tg_tag_irc_%s", tags[i] + 8);
+                    weechat_hashtable_set (extra_vars, str_temp, NULL);
                 }
             }
         }
@@ -267,19 +278,97 @@ trigger_callback_check_conditions (struct t_trigger *trigger,
 }
 
 /*
- * Replaces text using one or more regex in the trigger.
+ * Replaces text using regex.
+ *
+ * Returns: text replaced.
+ *
+ * Note: result must be freed after use.
+ */
+
+char *
+trigger_callback_regex_replace (struct t_trigger_context *context,
+                                const char *text,
+                                regex_t *regex,
+                                const char *replace)
+{
+    char *value;
+    struct t_hashtable *hashtable_options_regex;
+
+    if (!regex)
+        return NULL;
+
+    hashtable_options_regex = weechat_hashtable_new (
+        32,
+        WEECHAT_HASHTABLE_STRING,
+        WEECHAT_HASHTABLE_STRING,
+        NULL, NULL);
+
+    weechat_hashtable_set (context->pointers, "regex", regex);
+    weechat_hashtable_set (hashtable_options_regex,
+                           "regex_replace", replace);
+
+    value = weechat_string_eval_expression (
+        text,
+        context->pointers,
+        context->extra_vars,
+        hashtable_options_regex);
+
+    weechat_hashtable_free (hashtable_options_regex);
+
+    return value;
+}
+
+/*
+ * Translates chars.
+ *
+ * Returns: text with translated chars.
+ *
+ * Note: result must be freed after use.
+ */
+
+char *
+trigger_callback_regex_translate_chars (struct t_trigger_context *context,
+                                        const char *text,
+                                        const char *chars1,
+                                        const char *chars2)
+{
+    char *value, *chars1_eval, *chars2_eval;
+
+    chars1_eval = weechat_string_eval_expression (
+        chars1,
+        context->pointers,
+        context->extra_vars,
+        NULL);
+    chars2_eval = weechat_string_eval_expression (
+        chars2,
+        context->pointers,
+        context->extra_vars,
+        NULL);
+
+    value = weechat_string_translate_chars (text, chars1_eval, chars2_eval);
+
+    if (chars1_eval)
+        free (chars1_eval);
+    if (chars2_eval)
+        free (chars2_eval);
+
+    return value;
+}
+
+/*
+ * Executes regex commands.
  */
 
 void
-trigger_callback_replace_regex (struct t_trigger *trigger,
-                                struct t_trigger_context *context,
-                                int display_monitor)
+trigger_callback_regex (struct t_trigger *trigger,
+                        struct t_trigger_context *context,
+                        int display_monitor)
 {
     char *value;
     const char *ptr_key, *ptr_value;
     int i, pointers_allocated;
-    struct t_hashtable *hashtable_options_regex;
 
+    value = NULL;
     pointers_allocated = 0;
 
     if (trigger->regex_count == 0)
@@ -298,13 +387,16 @@ trigger_callback_replace_regex (struct t_trigger *trigger,
 
     for (i = 0; i < trigger->regex_count; i++)
     {
-        /* if regex is not set (invalid), skip it */
-        if (!trigger->regex[i].regex)
+        /* if regex is not set (invalid) for command "regex replace", skip it */
+        if ((trigger->regex[i].command == TRIGGER_REGEX_COMMAND_REPLACE)
+            && !trigger->regex[i].regex)
+        {
             continue;
+        }
 
         ptr_key = (trigger->regex[i].variable) ?
             trigger->regex[i].variable :
-            trigger_hook_regex_default_var[weechat_config_integer (trigger->options[TRIGGER_OPTION_HOOK])];
+            trigger_hook_regex_default_var[weechat_config_enum (trigger->options[TRIGGER_OPTION_HOOK])];
         if (!ptr_key || !ptr_key[0])
         {
             if (trigger_buffer && display_monitor)
@@ -336,25 +428,25 @@ trigger_callback_replace_regex (struct t_trigger *trigger,
             ptr_value = weechat_hashtable_get (context->extra_vars, ptr_key);
         }
 
-        hashtable_options_regex = weechat_hashtable_new (
-            32,
-            WEECHAT_HASHTABLE_STRING,
-            WEECHAT_HASHTABLE_STRING,
-            NULL, NULL);
-
-        weechat_hashtable_set (context->pointers,
-                               "regex", trigger->regex[i].regex);
-        weechat_hashtable_set (hashtable_options_regex,
-                               "regex_replace",
-                               trigger->regex[i].replace_escaped);
-
-        value = weechat_string_eval_expression (
-            ptr_value,
-            context->pointers,
-            context->extra_vars,
-            hashtable_options_regex);
-
-        weechat_hashtable_free (hashtable_options_regex);
+        switch (trigger->regex[i].command)
+        {
+            case TRIGGER_REGEX_COMMAND_REPLACE:
+                value = trigger_callback_regex_replace (
+                    context,
+                    ptr_value,
+                    trigger->regex[i].regex,
+                    trigger->regex[i].replace_escaped);
+                break;
+            case TRIGGER_REGEX_COMMAND_TRANSLATE_CHARS:
+                value = trigger_callback_regex_translate_chars (
+                    context,
+                    ptr_value,
+                    trigger->regex[i].str_regex,
+                    trigger->regex[i].replace);
+                break;
+            case TRIGGER_NUM_REGEX_COMMANDS:
+                break;
+        }
 
         if (value)
         {
@@ -493,7 +585,7 @@ trigger_callback_execute (struct t_trigger *trigger,
     if (weechat_trigger_plugin->debug >= 1)
     {
         gettimeofday (&(context->start_check_conditions), NULL);
-        context->start_replace_regex = context->start_check_conditions;
+        context->start_regex = context->start_check_conditions;
         context->start_run_command = context->start_check_conditions;
     }
 
@@ -505,7 +597,7 @@ trigger_callback_execute (struct t_trigger *trigger,
         /* replace text with regex */
         if (weechat_trigger_plugin->debug >= 1)
             gettimeofday (&(context->start_check_conditions), NULL);
-        trigger_callback_replace_regex (trigger, context, display_monitor);
+        trigger_callback_regex (trigger, context, display_monitor);
 
         /* execute command(s) */
         if (weechat_trigger_plugin->debug >= 1)
@@ -524,8 +616,8 @@ trigger_callback_execute (struct t_trigger *trigger,
         time_init = weechat_util_timeval_diff (&(context->start_exec),
                                                &(context->start_check_conditions));
         time_cond = weechat_util_timeval_diff (&(context->start_check_conditions),
-                                               &(context->start_replace_regex));
-        time_regex = weechat_util_timeval_diff (&(context->start_replace_regex),
+                                               &(context->start_regex));
+        time_regex = weechat_util_timeval_diff (&(context->start_regex),
                                                 &(context->start_run_command));
         time_cmd = weechat_util_timeval_diff (&(context->start_run_command),
                                               &(context->end_exec));
@@ -720,7 +812,6 @@ trigger_callback_modifier_cb (const void *pointer, void *data,
                               const char *modifier, const char *modifier_data,
                               const char *string)
 {
-    struct t_gui_buffer *buffer;
     const char *ptr_string;
     char *string_modified, *pos, *buffer_pointer;
     char *str_tags, **tags, *prefix, *string_no_color;
@@ -730,7 +821,7 @@ trigger_callback_modifier_cb (const void *pointer, void *data,
 
     TRIGGER_CALLBACK_CB_INIT(NULL);
 
-    buffer = NULL;
+    ctx.buffer = NULL;
     tags = NULL;
     num_tags = 0;
     string_no_color = NULL;
@@ -842,15 +933,15 @@ trigger_callback_modifier_cb (const void *pointer, void *data,
                 rc = sscanf (buffer_pointer, "0x%lx", &value);
                 if ((rc != EOF) && (rc != 0))
                 {
-                    buffer = (struct t_gui_buffer *)value;
+                    ctx.buffer = (struct t_gui_buffer *)value;
                     weechat_hashtable_set (
                         ctx.extra_vars,
                         "tg_plugin",
-                        weechat_buffer_get_string (buffer, "plugin"));
+                        weechat_buffer_get_string (ctx.buffer, "plugin"));
                     weechat_hashtable_set (
                         ctx.extra_vars,
                         "tg_buffer",
-                        weechat_buffer_get_string (buffer, "full_name"));
+                        weechat_buffer_get_string (ctx.buffer, "full_name"));
                     pos++;
                     if (pos[0])
                     {
@@ -877,12 +968,12 @@ trigger_callback_modifier_cb (const void *pointer, void *data,
                 free (buffer_pointer);
             }
         }
-        weechat_hashtable_set (ctx.pointers, "buffer", buffer);
+        weechat_hashtable_set (ctx.pointers, "buffer", ctx.buffer);
     }
 
     if (tags)
     {
-        if (!trigger_callback_set_tags (buffer, (const char **)tags, num_tags,
+        if (!trigger_callback_set_tags (ctx.buffer, (const char **)tags, num_tags,
                                         ctx.extra_vars))
         {
             goto end;
@@ -914,7 +1005,6 @@ trigger_callback_line_cb (const void *pointer, void *data,
                           struct t_hashtable *line)
 {
     struct t_hashtable *hashtable;
-    struct t_gui_buffer *buffer;
     struct t_weelist_item *ptr_item;
     unsigned long value;
     const char *ptr_key, *ptr_value;
@@ -943,9 +1033,9 @@ trigger_callback_line_cb (const void *pointer, void *data,
     rc = sscanf (ptr_value + 2, "%lx", &value);
     if ((rc == EOF) || (rc < 1))
         goto end;
-    buffer = (void *)value;
+    ctx.buffer = (void *)value;
 
-    weechat_hashtable_set (ctx.pointers, "buffer", buffer);
+    weechat_hashtable_set (ctx.pointers, "buffer", ctx.buffer);
     ptr_value = weechat_hashtable_get (line, "tags");
     tags = weechat_string_split ((ptr_value) ? ptr_value : "",
                                  ",",
@@ -981,7 +1071,7 @@ trigger_callback_line_cb (const void *pointer, void *data,
     if (string_no_color)
         free (string_no_color);
 
-    if (!trigger_callback_set_tags (buffer, (const char **)tags, num_tags,
+    if (!trigger_callback_set_tags (ctx.buffer, (const char **)tags, num_tags,
                                     ctx.extra_vars))
     {
         goto end;
@@ -1053,16 +1143,19 @@ end:
 int
 trigger_callback_print_cb  (const void *pointer, void *data,
                             struct t_gui_buffer *buffer,
-                            time_t date, int tags_count, const char **tags,
+                            time_t date, int date_usec,
+                            int tags_count, const char **tags,
                             int displayed, int highlight, const char *prefix,
                             const char *message)
 {
     char *str_tags, *str_tags2, str_temp[128], *str_no_color;
     int length;
-    struct tm *date_tmp;
+    struct timeval tv;
 
     TRIGGER_CALLBACK_CB_INIT(WEECHAT_RC_OK);
 
+    ctx.buffer = buffer;
+
     /* do nothing if the buffer does not match buffers defined in the trigger */
     if (trigger->hook_print_buffers
         && !weechat_buffer_match_list (buffer, trigger->hook_print_buffers))
@@ -1074,14 +1167,10 @@ trigger_callback_print_cb  (const void *pointer, void *data,
     /* add data in hashtables used for conditions/replace/command */
     trigger_callback_set_common_vars (trigger, ctx.extra_vars);
     weechat_hashtable_set (ctx.pointers, "buffer", buffer);
-    date_tmp = localtime (&date);
-    if (date_tmp)
-    {
-        if (strftime (str_temp, sizeof (str_temp),
-                      "%Y-%m-%d %H:%M:%S", date_tmp) == 0)
-            str_temp[0] = '\0';
-        weechat_hashtable_set (ctx.extra_vars, "tg_date", str_temp);
-    }
+    tv.tv_sec = date;
+    tv.tv_usec = date_usec;
+    weechat_util_strftimeval (str_temp, sizeof (str_temp), "%FT%T.%f", &tv);
+    weechat_hashtable_set (ctx.extra_vars, "tg_date", str_temp);
     snprintf (str_temp, sizeof (str_temp), "%d", displayed);
     weechat_hashtable_set (ctx.extra_vars, "tg_displayed", str_temp);
     snprintf (str_temp, sizeof (str_temp), "%d", highlight);
@@ -1143,6 +1232,8 @@ trigger_callback_command_cb  (const void *pointer, void *data,
     TRIGGER_CALLBACK_CB_NEW_POINTERS;
     TRIGGER_CALLBACK_CB_NEW_EXTRA_VARS;
 
+    ctx.buffer = buffer;
+
     /* add data in hashtables used for conditions/replace/command */
     trigger_callback_set_common_vars (trigger, ctx.extra_vars);
     weechat_hashtable_set (ctx.pointers, "buffer", buffer);
@@ -1194,6 +1285,8 @@ trigger_callback_command_run_cb  (const void *pointer, void *data,
     TRIGGER_CALLBACK_CB_NEW_POINTERS;
     TRIGGER_CALLBACK_CB_NEW_EXTRA_VARS;
 
+    ctx.buffer = buffer;
+
     /* add data in hashtables used for conditions/replace/command */
     trigger_callback_set_common_vars (trigger, ctx.extra_vars);
     weechat_hashtable_set (ctx.pointers, "buffer", buffer);
@@ -1217,8 +1310,7 @@ trigger_callback_timer_cb  (const void *pointer, void *data,
 {
     char str_temp[128];
     int i;
-    time_t date;
-    struct tm *date_tmp;
+    struct timeval tv_now;
 
     TRIGGER_CALLBACK_CB_INIT(WEECHAT_RC_OK);
 
@@ -1241,15 +1333,9 @@ trigger_callback_timer_cb  (const void *pointer, void *data,
     trigger_callback_set_common_vars (trigger, ctx.extra_vars);
     snprintf (str_temp, sizeof (str_temp), "%d", remaining_calls);
     weechat_hashtable_set (ctx.extra_vars, "tg_remaining_calls", str_temp);
-    date = time (NULL);
-    date_tmp = localtime (&date);
-    if (date_tmp)
-    {
-        if (strftime (str_temp, sizeof (str_temp),
-                      "%Y-%m-%d %H:%M:%S", date_tmp) == 0)
-            str_temp[0] = '\0';
-        weechat_hashtable_set (ctx.extra_vars, "tg_date", str_temp);
-    }
+    gettimeofday (&tv_now, NULL);
+    weechat_util_strftimeval (str_temp, sizeof (str_temp), "%FT%T.%f", &tv_now);
+    weechat_hashtable_set (ctx.extra_vars, "tg_date", str_temp);
 
     /* execute the trigger (conditions, regex, command) */
     if (!trigger_callback_execute (trigger, &ctx))
@@ -1300,6 +1386,8 @@ trigger_callback_focus_cb (const void *pointer, void *data,
 
     TRIGGER_CALLBACK_CB_NEW_POINTERS;
 
+    ctx.extra_vars = weechat_hashtable_dup (info);
+
     /* add data in hashtables used for conditions/replace/command */
     trigger_callback_set_common_vars (trigger, info);
     ptr_value = weechat_hashtable_get (info, "_window");
@@ -1436,5 +1524,8 @@ void
 trigger_callback_end ()
 {
     if (trigger_callback_hashtable_options_conditions)
+    {
         weechat_hashtable_free (trigger_callback_hashtable_options_conditions);
+        trigger_callback_hashtable_options_conditions = NULL;
+    }
 }
